"""Core filesystem operations - The Iron Vault kernel."""

# Async versions for non-blocking I/O
from .async_io import (
    atomic_write_async,
    copy_async,
    delete_async,
    digest_async,
    listdir_async,
    mkdir_async,
    move_async,
    read_async,
    read_bytes_async,
    shutdown_executor,
    write_async,
    write_bytes_async,
)
from .safeio import (
    # Configuration
    MAX_FILE_SIZE,
    MIN_FREE_DISK_SPACE,
    # Error types
    DiskSpaceError,
    FileSizeLimitError,
    atomic_write,
    check_disk_space,
    check_file_size,
    copy,
    delete,
    digest,
    listdir,
    mkdir,
    move,
    read,
    read_bytes,
    write,
    write_bytes,
)
from .sandbox import SandboxViolation, get_sandbox_root, resolve_safe_path

__all__ = [
    # Sync operations
    "atomic_write",
    "copy",
    "delete",
    "digest",
    "listdir",
    "mkdir",
    "move",
    "read",
    "read_bytes",
    "resolve_safe_path",
    "get_sandbox_root",
    "write",
    "write_bytes",
    # Async operations
    "atomic_write_async",
    "copy_async",
    "delete_async",
    "digest_async",
    "listdir_async",
    "mkdir_async",
    "move_async",
    "read_async",
    "read_bytes_async",
    "write_async",
    "write_bytes_async",
    "shutdown_executor",
    # Error types
    "DiskSpaceError",
    "FileSizeLimitError",
    "SandboxViolation",
    # Configuration
    "MAX_FILE_SIZE",
    "MIN_FREE_DISK_SPACE",
    "check_disk_space",
    "check_file_size",
]
