"""FastAPI service layer for The Iron Vault."""

from .app import app
from .auth import get_auth_status, require_auth
from .models import (
    DeleteIn,
    Entry,
    HashIn,
    ListIn,
    MkdirIn,
    MoveCopyIn,
    PathIn,
    ReadIn,
    WriteIn,
)

__all__ = [
    "app",
    "require_auth",
    "get_auth_status",
    "PathIn",
    "ListIn",
    "ReadIn",
    "WriteIn",
    "MoveCopyIn",
    "MkdirIn",
    "DeleteIn",
    "HashIn",
    "Entry",
]
